#!/usr/bin/perl -w
# add.pl
# David Rowe 4 March 2006
#
# Merges two PCB files by combining 'Element', 'Layer' and 'Via' entries

my $numArgs = $#ARGV + 1;
if ($numArgs != 3) {
    print "usage: ./add.pl inputPCBFile1 inputPCBFile1 outputPCBFile\n";
    exit;
}
my $input1 = shift;
my $input2 = shift;
my $output = shift;
my $in1;
my $in2;

open IN1, $input1 or die "Cant open file $_\n";
open IN2, $input2 or die "Cant open file $_\n";
open OUT, ">$output" or die "Cant open file $_\n";

# copy IN1 to out until we get to 'Layer(1', this
# is just after the last 'Element' in IN1.  This means we 
# use preamble and symbols from IN1

while ($in1 = <IN1>) {
    if ($in1 =~ /Layer\(1/) {
	last;
    }
    print OUT $in1;
}

# Skip the IN2 preamble up to the first 'Via' or 'Element'

while ($in2 = <IN2>) {
    if ($in2 =~ /Element\[/) {
	last;
    }
    if ($in2 =~ /Via\[/) {
	last;
    }
}

# Now copy IN2's Vias and Elements to OUT

print OUT $in2;
while ($in2 = <IN2>) {
    if ($in2 =~ /Layer\(1/) {
	last;
    }
    print OUT $in2;
}

# Now we need to combine the Lines in each Layer

do {

    if ($in1 =~ /NetList/) {
	close IN1;
	close IN2;
	close OUT;
	exit;
    }
    
    # IN1 lines 

    print OUT $in2;
    while ($in1 = <IN1>) {
	if ($in1 =~ /^\)/) {
	    last;
	}
	if ($in1 =~ /NetList/) {
	    close IN1;
	    close IN2;
	    close OUT;
	    exit;
	}
    
    	print OUT $in1;
    }

    
    # IN2 lines
    
    $in2 = <IN2>;
    while ($in2 = <IN2>) {
	if ($in2 =~ /Layer\(/) {
	    last;
	}
	if ($in2 =~ /NetList/) {
	    close IN1;
	    close IN2;
	    close OUT;
	    exit;
	}
	print OUT $in2;
    }
    
} while ($in1 = <IN1>);

close IN1;
close IN2;
close OUT;
