module xilinx_4fx(
  nCSA,               // assert this to talk to SPI devices
  nCSB,               // assert this to select SPI device
  SDI, 
  SCLK,
  A,                  // address we need to match for this card
  nCS4_0,             // 4 external chip selects
  LED_out             // two bits to each LED, allows bicolor operation
);

input        nCSA, nCSB, SDI, SCLK, A;
wire   [1:0] A;
output [4:0] nCS4_0;
wire   [4:0] nCS4_0;
output [7:0] LED_out;
wire   [7:0] LED_out;
wire         nCS;

// internal signals

wire   [7:0] sr1_out;  
wire         nCS5;     // chip select for internal LED register
wire         sr1_clk;
wire         sr2_clk;

assign sr1_clk = nCSB | SCLK;
assign nCS     = !(!nCSA & (sr1_out[7:6] == A));
assign sr2_clk = nCS5 | SCLK;

// register for nCSx selection
// format is D7 D6 D5 D4 D3 D2 D1 D0
//           A7 A6  X  X  X A2 A1 A0
// A[7:6] select this card from other cards on SPI bus
// A[2:0] select the SPI device on this card (only 5 devices decoded)
 
shift8 sr1(
.clk          (sr1_clk),
.serial_in    (SDI),
.parallel_out (sr1_out)
);

// decodes sr1 output to choose nCSx
// only valid if address bits A match

demux3x5 my_demux3x5(
.din    (nCS),
.select (sr1_out[2:0]),
.dout   ({nCS5,nCS4_0})
);

// register for LED control

shift8 sr2(
.clk          (sr2_clk),
.serial_in    (SDI),
.parallel_out (LED_out)
);

endmodule

