module xilinx_4fx_tb;
reg  SDI, nCSA, nCSB, SCLK;
reg  [1:0] A;
wire [4:0] nCS4_0;
wire [7:0] LED;
reg  [7:0] SPI_tx;
reg  [7:0] device;
reg  [7:0] passes;
reg        pass;

initial begin
  $dumpfile ("xilinx_4fx_tb.vcd");
  $dumpvars (1, my_xilinx_4fx);
  $dumpvars (2, my_xilinx_4fx.my_demux3x5);
  $display  ("SCLK\tnCSA\tnCSB\tSDI\tnCS4_0\tLED");
  $monitor  ("%b\t%b\t%b\t%b\t%b\t%b",
	    SCLK, nCSA, nCSB, SDI, nCS4_0, LED);
  SCLK = 1;
  SDI = 0;
  nCSA = 1;
  nCSB = 1;
  A = 2'b10;
  device = 0;
  passes = 0;

  repeat (6) begin
    // send byte to MUX via SPI (MSB first)

    #1 nCSB = 0;

    SPI_tx = (A << 6) + device;
    repeat (8) begin
      #1 SCLK = 0;
      SDI = SPI_tx[7];
      SPI_tx = SPI_tx << 1;
      #1 SCLK = 1;
    end	
  
    #1 nCSB = 1;

    // extra clock cycle with no CS asserted - make sure
    // nothing changes

    #1 SCLK = 0; 
    #1 SCLK = 1; 

    // now send byte to selected SPI device (LED register)

    #1 nCSA = 0; 

    SPI_tx = 8'b01000010;
    repeat (8) begin
      #1 SCLK = 0;
      SDI = SPI_tx[7];
      SPI_tx = SPI_tx << 1;
      #1 SCLK = 1;

      // device 0..4 test: check nCS is asserted on every rising SCLK edge

      if (nCS4_0 == (~(1<<device))) begin
        pass = 1;
      end
      else begin
        pass = 0;
      end
    end	

    if (device < 5) begin
      if (pass) begin
        $display("nCS%d TEST PASSES!", device);
        passes = passes + 1;
      end
      else begin
        $display("nCS%d TEST FAILED!", device);
      end
    end

    #1 nCSA = 1;

    // extra clock cycle with no CS asserted - make sure 
    // nothing changes

    #1 SCLK = 0; 
    #1 SCLK = 1; 

    #1

    // device 5 test: check LED register correct

    if (device == 5) begin
      if (LED == 8'b01000010) begin
        $display("LED TEST PASSES!");
        passes = passes + 1;
      end
      else begin
        $display("LED TEST FAILED!");
      end
    end

    device = device + 1;
  end
 
  $display("passes: %d/%d", passes, device);

end

xilinx_4fx my_xilinx_4fx(
  .nCSA    (nCSA), 
  .nCSB    (nCSB), 
  .SDI     (SDI), 
  .SCLK    (SCLK),
  .A       (A),
  .nCS4_0  (nCS4_0),
  .LED_out (LED)
);

endmodule
