#!/usr/bin/perl -w
# step.pl
# David Rowe 4 March 2006
#
# Makes a copy of a PCB file with a step y offset and
# a prefix added to component names.

my $numArgs = $#ARGV + 1;
if ($numArgs != 4) {
    print "usage: ./step.pl inputPCBFile outputPCBFile suffix step\n";
    print "  step is in units of 1E-5 inches\n";
    exit;
}
my $input = shift;
my $output = shift;
my $suffix = shift;
my $yStep = shift;
my %charWidth = ();

open IN, $input or die "Cant open file $_\n";
open OUT, ">$output" or die "Cant open file $_\n";

while (<IN>) {

    # build hash of all Symbol widths

    if (/Symbol\['(.*?)' (.*?)\]/) {
	$charWidth{$1} = $2;
    }
	
    if (/Element\[(".*?") (".*?") "(.*?)" (".*?") (.*?) (.*?) (.*?) (.*)\]/) {

	# work out width of suffix so we can center new text string
	@arr=split(//,$suffix);
	$textWidth = 0;
	foreach (@arr) {
	    $textWidth += $charWidth{$_};
	}

	#print "$_";
	$y = $6 + $yStep;
	$xText = $7 - $textWidth/2;
        #print "Element[$1 $2 \"$suffix/$3\" $4 $5 $y $xText $8]\n";
        print OUT "Element[$1 $2 \"$suffix$3\" $4 $5 $y $xText $8]\n";
    }
    elsif (/\sLine\[(.*?) (.*?) (.*?) (.*?) (.*)\]/) {
	#print "$_";
	$y1 = $2 + $yStep;
	$y2 = $4 + $yStep;
        #print "\tLine[$1 $y1 $3 $y2 $5]\n";
        print OUT "\tLine[$1 $y1 $3 $y2 $5]\n";
    }
    elsif (/\[(.*?) (.*?)\] \[(.*?) (.*?)\] \[(.*?) (.*?)\] \[(.*?) (.*?)\]/) {
	# handle rectangles, bit of a hack, depends on unqiue layout of
        # 4-coordate polygons.  Probably need a state machine to properly
        # handle Polygon defn which stretches over multiple lines and
	# possibly over 3-x points
	$y1 = $2 + $yStep;
	$y2 = $4 + $yStep;
	$y3 = $6 + $yStep;
	$y4 = $8 + $yStep;
        print OUT "\t\t[$1 $y1] [$3 $y2] [$5 $y3] [$7 $y4]\n";
    }
    elsif (/Via\[(.*?) (.*?) (.*)\]/) {
	$y1 = $2 + $yStep;
        print OUT "Via[$1 $y1 $3]\n";
    }
    else {
	print OUT $_;
    }
}

close IN;
close OUT;
