#!/usr/bin/perl -w
# subtract.pl
# David Rowe 6 March 2006
#
# De-merges two PCB files, opposite of add.pl

sub trim($);

my $numArgs = $#ARGV + 1;
if ($numArgs != 3) {
    print "usage: ./subtract.pl inputPCBFile1 inputPCBFile1 outputPCBFile\n";
    exit;
}
my $input1 = shift;
my $input2 = shift;
my $output = shift;
my $in1;
my $in2;

open IN1, $input1 or die "Cant open file $_\n";
open IN2, $input2 or die "Cant open file $_\n";
open OUT, ">$output" or die "Cant open file $_\n";

my $state = "normal";
my $nextState;

# construct hashes of all Elements, Lines, and vias in IN2

while (<IN2>) {
    $nextState = $state;

    if ($state =~ /normal/) {
	if (/Element\[/) {
	    $elements{$_} = 1;
	}    
	if (/\sLine\[/) {
	    $lines{$_} = 1;
	}
	if (/Via\[/) {
	    $vias{$_} = 1;
	}
	if (/Polygon/) {
	    $nextState = "polygon";
	}	    
    }
    
    if ($state =~ /polygon/) {
	if (/\[/) {
	    # store info unique to this polygon
	    # we assume all info on one line
	    $polygons{trim($_)} = 1;
	    #print "looking for poly: $_";
	}
    
	if (/\s\)/) {
	    $nextState = "normal";
	}
    }
    
    $state = $nextState;
}

# now delete any matching text in IN1

$state = "normal";
$line = 1;

while (<IN1>) {
    $nextState = $state;
    $line++;

    if ($state eq "normal") {
	if (defined $elements{$_}) {
	    $nextState = "element";
	}
	elsif (/Polygon/) {
	    $nextState = "polygon";
	}
	elsif (!defined $lines{$_} && !defined $vias{$_}) {
	    print OUT $_;
	}
    }

    if ($state eq "element") {
	if (/\s\)/) {
	    $nextState = "normal";
	}
    }

    if ($state eq "polygon") {
	# need state machine to handle polygons as we must defer
	# the decision until we get to the coordinate line 
	if (/\s\[/) {
	    #print "found poly: $_";
	    # this is the line defining the polygon
	    if (defined $polygons{trim($_)}) {
		# match - so print nothing
		#print "\tMATCH";
		$nextState = "polygonendnoprint";
	    }
	    else {
		# no match, print line in full, otherwise print nothing
		print OUT "\tPolygon(\"clearpoly\")\n\t(\n";
		print OUT $_;
		$nextState = "polygonendprint";
	    }
	}
    }

    if ($state eq "polygonendprint") {
	print OUT $_;
	if (/\s\)/) {
	    $nextState = "normal";
	}
    }

    if ($state eq "polygonendnoprint") {
	if (/\s\)/) {
	    $nextState = "normal";
	}
    }

    $state = $nextState;
}


close IN1;
close IN2;
close OUT;

sub trim($)
{
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}
